//--- Пространство имен приложения ---
var stdportal = stdportal || {};

stdportal.getItemIndex = function(array, name, value){
    var i;

    for(i = array.length; i--; ){
        if (value === array[i][name]){
            break;
        }
    }

    return i;
};

stdportal.getItem = function(array, name, value){
    var i;

    for(i = array.length; i--; ){
        if (value === array[i][name]){
            break;
        }
    }
    if (i < 0){
        return null;
    }

    return array[i];
};

//--- Создание пространства имен ---
stdportal.createNamespace = function (namespace) {
    var parts = namespace.split('.'),
        parent = stdportal,
        i,
        max;

    //--- Отбросить начальный префикс - имя глобального объекта ---
    if (parts[0] == 'stdportal') {
        parts = parts.slice(1);
    }

    for (i = 0, max = parts.length; i < max; i++) {
        //--- Создать свойство, если оно отсутствует ---
        if (typeof parent[parts[i]] === undefined) {
            parent[parts[i]] = {};
        }

        //--- Перейти на следующий уровень ---
        parent = parent[parts[i]];
    }
    return parent;
};

//--- Пространство имен плагинов ---
stdportal.createNamespace('plugins');

//--- Получение ссылки на конструктор плагина ---
stdportal.getPluginConstructor = function (type) {
    var parts = type.split('.'),
        parent = stdportal;
    //parent = stdportal.plugins;

    //--- Отбросить начальный префикс - stdportal ---
    if (parts[0] == 'stdportal') {
        parts = parts.slice(1);
    }

    //--- Отбросить начальный префикс - plugins ---
    /*if (parts[0] == 'plugins') {
        parts = parts.slice(1);
    }*/

    //--- Если объект не является конструктором, вывести сообщение об ошибке ---
    if (typeof parent[parts[0]] !== "function") {
        console.log('stdportal.getPluginConstructor: плагин "' + type + '" не существует');
        return null;
    }

    return parent[parts[0]];
};

stdportal.MixinDefaultValues = function(pluginVariables){
    var name,
        property,
        prototypeValue,
        instanceValue;

    for(name in pluginVariables){
        prototypeValue = this[name];

        if (typeof prototypeValue == 'object' && prototypeValue !== null){
            instanceValue = pluginVariables[name];
            if (typeof instanceValue == 'object' && instanceValue !== null){
                for(property in prototypeValue){
                    if (instanceValue[property] === undefined){
                        instanceValue[property] = prototypeValue[property];
                    }
                }
            }

            this[name] = instanceValue;
        }
        else{
            this[name] = pluginVariables[name];
        }
    }
};

//--- Получение ссылки на экземпляр плагина ---
stdportal.classes = {};
stdportal.getInstance = function (options, initialize) {
    var constr,//Конструктор плагина
        node,//DOM-узел
        instance = null,//Возвращаемый экземпляр
        varName,
        i,
        shortName,
        allOptions = stdportal.getInstance._options,
        typeOf = stdportal.typeOf,
        defaultValues,
        clsId,
        MixinDefaultValues = stdportal.MixinDefaultValues,
        varNameObject,
        method;//Метод для получения ссылки на экземпляр

    if (options === null){
        return options;
    }

    //--- Экземпляр плагина уже создан ---
    if (options.declaredClass !== undefined){
        return options;
    }

    //--- Идентификатор класса ---
    clsId = options.clsId;
    console.log('clsId=' + clsId);

    //--- Идентификатор класса отсутствует ---
    /*if (clsId === undefined){
        return options;
    }*/

    //--- Наследование значений по умолчанию ---
    if (!(options instanceof MixinDefaultValues)){
        //--- Значения по умолчанию ---
        defaultValues = stdportal.classes[clsId];

        if (defaultValues !== undefined){
            MixinDefaultValues.prototype = defaultValues;
            options =  new MixinDefaultValues(options);
        }
    }

    allOptions.push(options);

    //--- Получение ссылки на экземпляр путем вызова метода ---
    if (options.method !== undefined){
        if (options.varName !== undefined){
            varName = options.varName;
            for(i = allOptions.length - 1; i--; ){
                instance = allOptions[i][varName];
                if (instance !== undefined && instance !== null &&
                    (typeOf(instance) != 'object' || instance.declaredClass !== undefined)){
                    break;
                }
            }
            if (i < 0){
                console.log('stdportal.getInstance: экземпляр varName="' + options.varName + '" не существует');
                allOptions.pop();
                return null;
            }

            delete options.varName;
        }
        else if (options.id !== undefined){
            //--- Экземпляр ---
            instance = dijit.byId(options.id);
            if (instance === null) {
                console.log('stdportal.getInstance: экземпляр с id="' + options.id + '" не существует');
                allOptions.pop();
                return null;
            }
            delete options.id;
        }

        method = options.method;
        delete options.method;
        instance = instance[method](options);
    }
    else if (options.varName !== undefined){
        varName = options.varName;
        for(i = allOptions.length - 1; i--; ){
            instance = allOptions[i][varName];
            if (instance !== undefined && instance !== null &&
                (typeOf(instance) != 'object' || instance.declaredClass !== undefined)){
                break;
            }

            varNameObject = allOptions[i]._varName;
            if (varNameObject !== undefined){
                //delete options._varName;

                instance = varNameObject[varName];
                if (instance !== undefined && instance !== null &&
                    (typeOf(instance) != 'object' || instance.declaredClass !== undefined)){
                    break;
                }
            }
        }
        if (i < 0){
            console.log('stdportal.getInstance: экземпляр varName="' + options.varName + '" не существует');
            allOptions.pop();
            return null;
        }

        //delete options.varName;
    }
    else if (options.clsId !== undefined){
        //--- Статический класс ---
        if (options.static){
            //--- Короткое имя класса ---
            shortName = options.clsId;
            //shortName = shortName.substr(shortName.lastIndexOf('.') + 1);

            instance = stdportal[shortName];

            if (typeof instance !== 'object'){
                console.log('stdportal.getInstance: статический класс ' + options.clsId +
                    ' имеет тип ' + typeof stdportal[options.clsId] +
                    ' (должен быть тип object)'
                );
                allOptions.pop();
                return null;
            }
        }
        //--- Создание экземпляра ---
        else{
            //--- Конструктор плагина ---
            constr = stdportal.getPluginConstructor(options.clsId);
            if (constr === null) {
                allOptions.pop();
                return instance;
            }

            //--- Создание экземпляра на основе существующего DOM-узла ---
            if (options.domId !== undefined){
                //--- DOM-узел экземпляра ---
                node = dojo.byId(options.domId);
                if (node === null) {
                    console.log('stdportal.getInstance: DOM-узел с id="' + options.domId + '" не существует');
                    allOptions.pop();
                    return instance;
                }
                delete options.domId;

                //--- Создание экземпляра ---
                instance = new constr(options, node);
            }
            //--- Добавление экземпляра перед существующим DOM-узлом ---
            else if (options.insertBefore !== undefined){
                //--- DOM-узел контейнера ---
                node = dojo.byId(options.insertBefore);
                if (node === null) {
                    console.log('stdportal.getInstance: DOM-узел с id="' + options.insertBefore + '" не существует');
                    allOptions.pop();
                    return instance;
                }
                delete options.insertBefore;

                //--- Создание экземпляра ---
                instance = new constr(options);
                node.parentNode.insertBefore(instance.domNode, node);
            }
            //--- Добавление экземпляра в существующий контейнер ---
            else if (options.containerId !== undefined){
                //--- DOM-узел контейнера ---
                node = dojo.byId(options.containerId);
                if (node === null) {
                    console.log('stdportal.getInstance: DOM-узел с id="' + options.containerId + '" не существует');
                    allOptions.pop();
                    return instance;
                }
                delete options.containerId;

                //--- Создание экземпляра ---
                instance = new constr(options);
                node.appendChild(instance.domNode);
            }
            //--- Добавление экземпляра в контейнеры родительского плагина ---
            else if (options.containerName !== undefined){
                /*containerName = options.containerName;

                for(i = allOptions.length; i--; ){
                    node = allOptions[containerName];
                    if (node !== undefined){
                        break;
                    }
                }
                if (i < 0){
                    console.log('stdportal.getInstance: экземпляр containerName="' + options.containerName + '" не существует');
                    allOptions.pop();
                    return null;
                }

                delete options.containerName;

                //--- Создание экземпляра ---
                instance = new constr(options);
                node.appendChild(instance.domNode);*/
                instance = new constr(options);
            }
            //--- Создание экземпляра без привязки к DOM-узлу ---
            else{
                instance = new constr(options);
            }

            //console.log('/clsId=' + clsId + '--------------------------------------------------');
            //--- Инициализация экземпляра ---
            if (initialize && options._init !== false){
                //console.log('clsId=' + clsId + ' init');
                instance.init(options, stdportal.PortalBase.config);
            }
            //--- Инициализация экземпляра ---
            /*if (options.init !== false){
                if ((instance.init(options, stdportal.PortalBase.config)) === null){
                    allOptions.pop();
                    return null;
                }
            }*/
        }
    }

    //--- Получение ссылки на экземпляр ---
    else if (options.id !== undefined){
        //--- Экземпляр ---
        instance = dijit.byId(options.id);
        if (!instance) {
            console.log('stdportal.getInstance: экземпляр с id="' + options.id + '" не существует');
            allOptions.pop();
            return null;
        }
        delete options.id;
    }

    //--- Отсутствует способ получения ссылки на экземпляр ---
    else{
        console.log('stdportal.getInstance: не указан ни один из способов получения ссылки на экземпляр');
        allOptions.pop();
        return instance;
    }

    allOptions.pop();
    //console.log('allOptions.length=' + allOptions.length);
    console.log('----------- clsId=' + clsId);
    return instance;
};
stdportal.getInstance._options = [];

//--- Определение типа переменной ---
stdportal.typeOf = function (variable) {
    var type = typeof variable;

    if (type == 'object' && dojo.isArray(variable)){
        type = 'array';
    }

    return type;
};

//--- Определение типа у свойств объекта ---
stdportal.getTypeOfObject = function (variable) {
    var type = null,
        response,
        //hasService = false,
        name,
        value;

    for(name in variable){
        if (!variable.hasOwnProperty(name)){
            continue;
        }

        //--- Закрытые переменные и методы ---
        if (name.indexOf('_') === 0){
            //--- В открытой переменной не могут присутствовать закрытые свойства ---
            delete variable.name;

            continue;
        }

        //--- Значение переменной ---
        value = variable[name];

        //--- Тип переменной ---
        type = stdportal.typeOf(value);

        //--- Отсеиваем свойства с неизвестным типом ---
        if (type == 'undefined'){
            delete variable.name;

            type = null;

            continue;
        }

        //--- Методы ---
        if (type == 'function'){
            //--- В открытой переменной не могут передаваться функции ---
            delete variable.name;

            type = null;

            continue;
        }

        //--- Строковый тип ---
        if (type == 'string'){
            //--- Плагин ---
            if (value == 'plugin'){
                type = 'plugin';
            }
            //--- Сервис ---
            else if (value == 'service'){
                type = 'service';
                //hasService = true;
            }
        }
        //--- Массив ---
        else if (type == 'array'){
            if (value.length == 1){
                //--- Массив плагинов ---
                if (value[0] == 'plugin'){
                    type = value;
                }
                //--- Массив сервисов ---
                else if (value[0] == 'service'){
                    type = value;
                }
            }
        }
        //--- Объект ---
        else if (type == 'object'){
            response = stdportal.getTypeOfObject(value);

            //--- Определены типы свойств объекта ---
            if (response !== null){
                //--- Одно из свойств объекта имеет тип "Сервис" ---
                if (response === true){
                    //hasService = true;
                }

                continue;
            }
        }

        variable[name] = type;
    }

    //--- Объект не имеет свойств ---
    if (type === null){
        response = null;
    }
    else{
        //response = hasService;
    }

    return response;
};

//--- Проверка и инициализация конфигурационных переменных ---
stdportal.initVariables = function (pluginConfig, configuration) {
    var /*configuration = this._configuration,//Конфигурационные переменные
        */servicesArray = portal.config.services,
        pageName = portal.pageName,
        getInstance = stdportal.getInstance,
        allOptions = getInstance._options,
        varName,
        inheritService,
        serviceOptions,
        serviceConfig,
        serviceId,
        services,
        service,
        plugins,
        plugin,
        description,
        value,
        name,
        type,
        isWrongType,
        descriptionType,
        temp,
        i,
        max,
        defaultValues = stdportal.classes[pluginConfig.clsId],
        typeOf = stdportal.typeOf,
        goodCheck = true;//Аварийное уничтожение экземпляра

    if (this === stdportal) {
        console.log('FATAL (stdportal.checkVariables): this ссылается на stdportal');
        return false;
    }

    //--- Проверка конфигурационных переменных ---
    for(name in configuration){
        if (!configuration.hasOwnProperty(name)){
            continue;
        }

        isWrongType = false;

        //--- Описание переменной ---
        description = configuration[name];

        //--- Тип переменной ---
        descriptionType = description.type;

        //--- Значение переменной ---
        value = pluginConfig[name];

        //--- В конфигурации не задано значение переменной ---
        //if (value === undefined){
            /*//--- Обязательная переменная не проинициализирована ---
            if (description.required){
                goodCheck = false;//Аварийное уничтожение экземпляра

                console.log('FATAL (' + this.declaredClass + '.init): в экземпляре с id="' +
                    + this.id + '" обязательная переменная ' +
                    name + ' не проинициализирована'
                );
            }
            //--- Присваиваем значение по умолчанию ---
            else{*/
                //value = description.default;
                //--- Клонирование !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1
                //pluginConfig[name] = description.default;
            //}
        //}
        /*//--- Проверка типа ---
        else{*/
            //type = typeOf(value);

            switch(descriptionType){
                case 'plugin':
                    switch(typeOf(value)){
                        //--- Значение задано ---
                        case 'object':
                            pluginConfig[name] = getInstance(value, true);
                            break;

                        //--- Значение не задано ---
                        //--- Неверное значение ---
                        default:
                            if (pluginConfig.hasOwnProperty(name)){
                                delete pluginConfig[name];
                            }
                            else if (defaultValues.hasOwnProperty(name)){
                                delete defaultValues[name];
                            }

                            isWrongType = true;

                            break;
                    }

                    break;

                case 'service':
                    switch(typeOf(value)){
                        //--- Значение задано ---
                        case 'object':
                            if (value !== null){
                                //--- Сервис еще не прикреплен ---
                                if (value.service === undefined){
                                    if (value.varName !== undefined){
                                        varName = value.varName;
                                        for(i = allOptions.length - 1; i--; ){
                                            inheritService = allOptions[i][varName];
                                            if (inheritService !== undefined && inheritService.service !== undefined){
                                                break;
                                            }
                                        }
                                        if (i < 0){
                                            console.log('stdportal.initVariables: сервис varName="' + value.varName + '" не существует');
                                            isWrongType = true;

                                            break;
                                        }

                                        delete value.varName;

                                        value.service = inheritService.service;
                                    }
                                    else{
                                        //--- Идентификатор сервиса ---
                                        serviceId = value.id;

                                        //--- Идентификатор сервиса не указан ---
                                        if (serviceId === undefined){
                                            isWrongType = true;

                                            break;
                                        }

                                        //--- Конфигурация сервиса ---
                                        serviceConfig = sovzond.byId(servicesArray, serviceId);

                                        //--- Конфигурация сервиса отсутствует ---
                                        if (serviceConfig === undefined){
                                            isWrongType = true;

                                            break;
                                        }

                                        //value._config = serviceConfig;
                                        value.service = serviceConfig.esriService;
                                    }
                                }

                                break;
                            }

                        //--- Значение не задано ---
                        //--- Неверное значение ---
                        default:
                            if (pluginConfig.hasOwnProperty(name)){
                                delete pluginConfig[name];
                            }
                            else if (defaultValues.hasOwnProperty(name)){
                                delete defaultValues[name];
                            }

                            isWrongType = true;

                            break;
                    }

                    break;

                default:
                    temp = typeOf(descriptionType);

                    //--- Типизированный массив ---
                    if (temp == 'array' && descriptionType.length !== 0){
                        //--- Значение не является массивом ---
                        if (typeOf(value) !== 'array'){
                            value = [];
                        }

                        switch(descriptionType[0]){
                            case 'plugin':
                                //--- Массив плагинов ---
                                plugins = [];
                                for(i = 0, max = value.length; i < max; i++){
                                    plugin = value[i];

                                    //--- Экземпляр плагина еще не создан ---
                                    //if (plugin.declaredClass === undefined){
                                        //--- Присвоение переменной ссылки на другой экземпляр ---
                                        plugin = getInstance(plugin, true);
                                    //}

                                    if (plugin === null){
                                        continue;
                                    }

                                    plugins.push(plugin);
                                }

                                //--- Плагины присутствуют ---
                                if (plugins.length !== 0){
                                    pluginConfig[name] = plugins;
                                    break;
                                }

                            case 'service':
                                //--- Массив сервисов ---
                                services = [];
                                for(i = value.length; i--; ){
                                    serviceOptions = value[i];

                                    //--- Значение не является конфигурацией сервиса ---
                                    if (typeOf(serviceOptions) !== 'object' || serviceOptions === null){
                                        value.splice(i, 1);
                                        continue;
                                    }

                                    //--- Сервис еще не прикреплен ---
                                    if (serviceOptions.service === undefined){
                                        if (serviceOptions.varName !== undefined){
                                            varName = serviceOptions.varName;
                                            for(i = allOptions.length - 1; i--; ){
                                                inheritService = allOptions[i][varName];
                                                if (inheritService !== undefined && inheritService.service !== undefined){
                                                    break;
                                                }
                                            }
                                            if (i < 0){
                                                console.log('stdportal.initVariables: сервис varName="' + serviceOptions.varName + '" не существует');
                                                value.splice(i, 1);

                                                continue;
                                            }

                                            delete serviceOptions.varName;

                                            serviceOptions.service = inheritService.service;
                                        }
                                        else{
                                            //--- Идентификатор сервиса ---
                                            serviceId = serviceOptions.id;

                                            //--- Идентификатор сервиса не указан ---
                                            if (serviceId === undefined){
                                                value.splice(i, 1);

                                                continue;
                                            }

                                            //--- Конфигурация сервиса ---
                                            serviceConfig = sovzond.byId(servicesArray, serviceId);

                                            //--- Конфигурация сервиса отсутствует ---
                                            if (serviceConfig === undefined){
                                                value.splice(i, 1);

                                                continue;
                                            }

                                            serviceOptions.service = serviceConfig.esriService;
                                        }
                                    }
                                }

                                //--- Сервисы отсутствуют ---
                                if (value.length === 0){
                                    pluginConfig[name] = dojo.clone(this[name]);
                                }
                                //--- Сервисы присутствуют ---
                                else{
                                    break;
                                }

                            //--- Значение не задано или неверно ---
                            default:
                                if (pluginConfig.hasOwnProperty(name)){
                                    delete pluginConfig[name];
                                }
                                else if (defaultValues.hasOwnProperty(name)){
                                    delete defaultValues[name];
                                }

                                isWrongType = true;
                                break;
                        }
                    }
                    else if (temp == 'object'){

                    }
                    else{
                        //--- Значение не задано ---
                        //--- Значение неверного типа ---
                        if (value === undefined || typeOf(value) !== descriptionType){
                            if (pluginConfig.hasOwnProperty(name)){
                                delete pluginConfig[name];
                            }
                            else if (defaultValues !== undefined && defaultValues.hasOwnProperty(name)){
                                delete defaultValues[name];
                            }

                            switch(descriptionType){
                                case 'string':
                                case 'number':
                                case 'boolean':
                                    break;

                                case 'object':
                                    value = this[name];
                                    if (value !== null){
                                        pluginConfig[name] = dojo.clone(value);
                                    }

                                    break;

                                case 'array':
                                    pluginConfig[name] = dojo.clone(this[name]);

                                    break;

                                default:
                                    break;
                            }

                            isWrongType = true;
                        }

                        //--- Неверное или отсутствующее значение ---
                        if (isWrongType){

                            /*//--- Значение по умолчанию ---
                             switch(descriptionType){
                                 case 'object':
                                     value = null;
                                     pluginConfig[name] = null;
                                     break;

                                 case 'array':
                                     value = [];
                                     pluginConfig[name] = [];
                                     break;

                                 case 'string':
                                 case 'number':
                                 case 'boolean':
                                     delete pluginConfig[name];
                                     break;

                                 default:
                                     console.log('ATTENTION (' + this.declaredClass + '.init): в экземпляре с id="' +
                                         + this.id + '" обязательная переменная ' +
                                         name + 'имеет недопустимый тип ' +
                                         descriptionType
                                     );
                                     break;
                             }*/
                        }
                    }

                    break;
            }
        //}

        if (isWrongType){
            //--- Обязательная переменная ---
            if (description.required){
                goodCheck = false;//Аварийное уничтожение экземпляра

                console.log('FATAL (' + this.declaredClass + '.init): в экземпляре с id="' +
                    + this.id + '" обязательной переменной ' +
                    name + ', имеющей тип ' +
                    descriptionType + ' присвоено значение другого типа (' + typeOf(value) + ')'
                );

                break;
            }
            //--- Присваиваем значение по умолчанию ---
            else{
                //value = description.default;
                //pluginConfig[name] = description.default;

                /*console.log('ATTENTION (' + this.declaredClass + '.init): в экземпляре с id="' +
                    + this.id + '" обязательной переменной ' +
                    name + ', имеющей тип ' +
                    descriptionType + ' присвоено значение другого типа (' + (value === null ? null : typeOf(value)) + ')'
                );*/
            }
        }

        //this[name] = value;
    }

    return goodCheck;
};

stdportal.CreateInheritance = function(inheritObject){
    function F() {
    }

    F.prototype = inheritObject;
    return new F();
};

//--- Объявление класса плагина ---
stdportal.declare = function (declaredClass, inheriteDijit, options) {
    var configuration = {},
        name,
        value,
        type,
        i,
        required,
        shortName = '',
        strings,
        response,
        customConstructor,
        typeOf = stdportal.typeOf,
        CreateInheritance = stdportal.CreateInheritance,
        isStrings = false,
        hasService = false;

    //--- Короткое имя класса ---
    shortName = declaredClass.substr(declaredClass.lastIndexOf('.') + 1);
    if (shortName === ''){
        console.log('FATAL (stdportal.declare): неверное имя класса ' + declaredClass);
        return;
    }

    //--- Объект строковых констант ---
    if (options.hasOwnProperty('_strings')){
        strings = options._strings;

        if (typeOf(strings) != 'object' || strings === null){
            delete options._strings;
            strings = undefined;
        }
    }

    //--- Наследование других плагинов ---
    if (inheriteDijit !== null && inheriteDijit.length > 1){
        options._strings = null;
    }

    //--- Функция constructor ---
    customConstructor = options.constructor;
    if (typeof customConstructor !== 'function'){
        customConstructor = function(){
        };
    }
    options._constructor = options.constructor;
    options.constructor = function(pluginConfig, dom){
        var thisString = this._strings,
            tempStrings,
            prototypeObject,
            name;

        //--- Проверка и инициализация конфигурационных переменных ---
        if (!stdportal.initVariables.call(this, pluginConfig, configuration)){
            console.log('FATAL (stdportal.declare): инициализация экземпляра id=' + this.id +
                ' класса ' + declaredClass + ' не удалась');
            //--- Инициализация не удалась ---
            if (this.destroyRecursive) {
                this.destroyRecursive(true);
            }
            delete this;
        }

        if (thisString === null){
            prototypeObject = this.constructor.prototype;
            if (prototypeObject.hasOwnProperty('_strings')){
                tempStrings = this.__strings;

                switch(pluginConfig.clsId){
                    //--- Объект принадлежит текущему классу ---
                    case shortName:
                        //--- Унаследованные строки отсутствуют ---
                        if (tempStrings === undefined){
                            //--- Собственные строки отсутствуют ---
                            if (strings === undefined){
                                delete prototypeObject._strings
                            }
                            //--- Собственные строки присутствуют ---
                            else{
                                prototypeObject._strings = strings;
                            }
                        }
                        //--- Унаследованные строки присутствуют ---
                        else{
                            //--- Собственные строки присутствуют ---
                            if (strings !== undefined) {
                                for (name in strings) {
                                    tempStrings[name] = strings[name];
                                }
                            }

                            prototypeObject._strings = tempStrings;
                            delete prototypeObject.__strings;
                        }

                        break;

                    default:
                        //--- Собственные строки присутствуют ---
                        if (strings !== undefined){
                            if (tempStrings === undefined){
                                prototypeObject.__strings = CreateInheritance(strings);
                            }
                            else{
                                for(name in strings){
                                    tempStrings[name] = strings[name];
                                }
                            }
                        }
                }
            }
        }

        //--- Пользовательская функция constructor ---
        customConstructor.call(this, pluginConfig, dom);

        this._config = pluginConfig;
    };

    //--- Пользовательская функция инициализации ---
    if (typeof options.init !== 'function'){
        options.init = function(){

        };
    }

    //--- Функция postCreate ---
    if (typeof options.postCreate !== 'function'){
        options.postCreate = function(){};
    }
    options._postCreate = options.postCreate;
    options.postCreate = function(){
        //--- Пользовательская функция postCreate ---
        this._postCreate();

        //--- Базовый класс CSS ---
        dojo.addClass(this.domNode, shortName);
        //this.domNode.className = shortName;

        //--- Пользовательская функция инициализации ---
        //this.init(portal.config);
    };

    //--- Конфигурационные переменные ---
    for(name in options){
        if (!options.hasOwnProperty(name)){
            continue;
        }

        //--- Закрытые переменные и методы ---
        if (name.indexOf('_') === 0){
            continue;
        }

        //--- Значение по умолчанию открытой переменной ---
        value = options[name];

        //--- Тип открытой переменной ---
        type = typeOf(value);

        //--- Отсеиваем открытые переменные с неизвестным типом ---
        if (type == 'undefined'){
            delete options.name;
            continue;
        }

        //--- Открытые методы ---
        if (type == 'function'){
            continue;
        }

        //--- Строковый тип ---
        if (type == 'string'){
            //--- Плагин ---
            if (value == 'plugin'){
                type = 'plugin';
                value = null;
                options[name] = null;
            }
            //--- Сервис ---
            else if (name == 'service'/*value == 'service'*/){
                //hasService = true;
                type = 'service';
                value = null;
                options[name] = null;
            }
        }
        //--- Массив ---
        else if (type == 'array'){
            if (value.length == 1){
                //--- Массив плагинов ---
                if (value[0] == 'plugin'){
                    type = value;
                    value = [];
                    options[name] = [];
                }
                //--- Массив сервисов ---
                else if (value[0] == 'service'){
                    //hasService = true;
                    type = value;
                    value = [];
                    options[name] = [];
                }
            }
        }
        //--- Объект ---
        else if (type == 'object'){
            /*if (value !== null){
                response = stdportal.getTypeOfObject(value);

                //--- Определены типы свойств объекта ---
                if (response !== null){
                    //--- Одно из свойств объекта имеет тип "Сервис" ---
                    //if (response === true){
                        //hasService = true;
                    //}

                    type = value;
                }

                value = null;
                options[name] = null;
            }*/
        }

        /*if (type == 'array' && value[0] == 'service'){
            type = 'service';
            value = [];
        }*/

        //--- Добавление конфигурационной переменной в реестр ---
        configuration[name] = {
            type: type,
            required: false
        };

        delete options.name;
        //--- Начальное значение открытой переменной ---
        //options[name] = undefined;
    }

    //--- Обязательные переменные ---
    required = options._required;
    if (dojo.isArray(required)){
        for(i = required.length; i--; ){
            name = required[i];

            if (configuration[name] !== undefined){
                configuration[name].required = true;
            }
        }

    }

    //--- Реестр конфигурационных переменных ---
    //options._configuration = configuration;

    if (options.static){
        stdportal[shortName] = options;
        return;
    }

    //--- Вызов функции dojo.declare ---
    dojo.declare(declaredClass, inheriteDijit, options);
};

//--- Ограничение размеров узла ---
stdportal.onResize = function(constraints){
    var body = document.body,
        bodySize = {
            width: dojo.style(body, 'width'),
            height: dojo.style(body, 'height')
        },
        node = this,
        i,
        k,
        part,
        constraintsParts,
        /*constraints = [
            {
                size: 'width',
                name: 'minWidth',
                parts: []//Обратный порядок
            }
        ],*/
        constraintsOptions,
        size;

    if (node === undefined){
        node = this.domNode;
    }

    //--- Ограничения размеров ---
    for(i = constraints.length; i--; ){
        constraintsOptions = constraints[i];

        size = 0;
        constraintsParts = constraintsOptions.parts;
        for(k = constraintsParts.length; k--; ){
            part = constraintsParts[k];

            switch(part.unit){
                case '%':
                    size += bodySize[constraintsOptions.name[3] == 'W' ? 'width' : 'height'] * part.value;
                    break;

                case 'px':
                    size += part.value;
                    break;

                default:
                    break;
            }
        }

        node.style[constraintsOptions.name] = (size === 0 ? 'none' : size + 'px');
    }
};
stdportal.rusToEng = function (rus) {
    var i, //Переменная цикла
        rusToEng = stdportal.rusToEng,
        temp; //Временный массив из фрагментов строки

    i = rus.length;
    temp = new Array(i);
    for (i = rus.length; i--;)
        if (rus[i] in rusToEng.iso)
            temp[i] = rusToEng.iso[rus[i]];
        else
            temp[i] = rus[i];

    return temp.join('');
};
stdportal.rusToEng.iso = { "Є":"YE", "І":"I", "Ѓ":"G", "і":"i", "№":"#", "є":"ye", "ѓ":"g", "А":"A", "Б":"B", "В":"V", "Г":"G", "Д":"D", "Е":"E", "Ё":"YO", "Ж":"ZH", "З":"Z", "И":"I", "Й":"J", "К":"K", "Л":"L", "М":"M", "Н":"N", "О":"O", "П":"P", "Р":"R", "С":"S", "Т":"T", "У":"U", "Ф":"F", "Х":"X", "Ц":"C", "Ч":"CH", "Ш":"SH", "Щ":"SHH", "Ъ":"'", "Ы":"Y", "Ь":"", "Э":"E", "Ю":"YU", "Я":"YA", "а":"a", "б":"b", "в":"v", "г":"g", "д":"d", "е":"e", "ё":"yo", "ж":"zh", "з":"z", "и":"i", "й":"j", "к":"k", "л":"l", "м":"m", "н":"n", "о":"o", "п":"p", "р":"r", "с":"s", "т":"t", "у":"u", "ф":"f", "х":"x", "ц":"c", "ч":"ch", "ш":"sh", "щ":"shh", "ъ":"", "ы":"y", "ь":"", "э":"e", "ю":"yu", "я":"ya", "«":"", "»":"", "—":"-" };
stdportal.rusToEng.iso[" "] = "_";
stdportal.rusToEng.iso["-"] = "_";
